@@ -75,51 +75,6 @@ let escape_res_name name allow_dirs =
 		else
 			"-x" ^ (string_of_int (Char.code chr))) name
 
-let update_cache_dependencies t =
-	let visited_anons = ref [] in
-	let rec check_t m t = match t with
-		| TInst(c,tl) ->
-			add_dependency m c.cl_module;
-			List.iter (check_t m) tl;
-		| TEnum(en,tl) ->
-			add_dependency m en.e_module;
-			List.iter (check_t m) tl;
-		| TType(t,tl) ->
-			add_dependency m t.t_module;
-			List.iter (check_t m) tl;
-		| TAbstract(a,tl) ->
-			add_dependency m a.a_module;
-			List.iter (check_t m) tl;
-		| TFun(targs,tret) ->
-			List.iter (fun (_,_,t) -> check_t m t) targs;
-			check_t m tret;
-		| TAnon an ->
-			if not (List.memq an !visited_anons) then begin
-				visited_anons := an :: !visited_anons;
-				PMap.iter (fun _ cf -> check_field m cf) an.a_fields
-			end
-		| TMono r ->
-			(match r.tm_type with
-			| Some t -> check_t m t
-			| _ -> ())
-		| TLazy f ->
-			check_t m (lazy_type f)
-		| TDynamic t ->
-			if t == t_dynamic then
-				()
-			else
-				check_t m t
-	and check_field m cf =
-		check_t m cf.cf_type
-	in
-	match t with
-		| TClassDecl c ->
-			List.iter (check_field c.cl_module) c.cl_ordered_statics;
-			List.iter (check_field c.cl_module) c.cl_ordered_fields;
-			(match c.cl_constructor with None -> () | Some cf -> check_field c.cl_module cf);
-		| _ ->
-			()
-
 (* -------------------------------------------------------------------------- *)
 (* FIX OVERRIDES *)
 
@@ -169,7 +124,7 @@ let fix_override com c f fd =
 					   have to detect this case and change the variable (issue #2712). *)
 					begin match follow v.v_type with
 						| TInst({cl_kind = KTypeParameter [tc]} as cp,_) when com.platform = Flash ->
-							if List.mem_assoc (snd cp.cl_path) c.cl_params then raise (Unify_error [])
+							if List.exists (fun tp -> tp.ttp_name = (snd cp.cl_path)) c.cl_params then raise (Unify_error [])
 						| _ ->
 							()
 					end;
@@ -257,7 +212,7 @@ let rec is_volatile t =
 		is_volatile (lazy_type f)
 	| TType (t,tl) ->
 		(match t.t_path with
-		| _ -> is_volatile (apply_params t.t_params tl t.t_type))
+		| _ -> is_volatile (apply_typedef t tl))
 	| _ ->
 		false
 
