@@ -30,7 +30,7 @@ let add_static c cf =
 let add_field c cf override =
 	c.cl_fields <- PMap.add cf.cf_name cf c.cl_fields;
 	c.cl_ordered_fields <- cf :: c.cl_ordered_fields;
-	if override then c.cl_overrides <- cf :: c.cl_overrides
+	if override then add_class_field_flag cf CfOverride
 
 let add_meta com en cl_enum =
 	Option.may (fun expr ->
@@ -54,7 +54,7 @@ module EnumToClass2Modf = struct
 		(* create the class *)
 		let cl_enum = mk_class en.e_module en.e_path pos in
 		cl_enum.cl_super <- Some (base_class,[]);
-		cl_enum.cl_extern <- en.e_extern;
+		if en.e_extern then add_class_flag cl_enum CExtern;
 		cl_enum.cl_meta <- [(Meta.Enum,[],pos); (Meta.NativeGen,[],pos)] @ cl_enum.cl_meta;
 
 		(* mark the enum that it's generated as a class *)
@@ -94,7 +94,7 @@ module EnumToClass2Modf = struct
 			let pos = ef.ef_pos in
 
 			let cl_ctor = mk_class en.e_module (e_pack, e_name ^ "_" ^ name) pos in
-			cl_ctor.cl_final <- true;
+			add_class_flag cl_ctor CFinal;
 			cl_ctor.cl_super <- Some (cl_enum, []);
 			cl_ctor.cl_meta <- [
 				(Meta.Enum,[],pos);
