@@ -1,6 +1,6 @@
 open Globals
 open Common
-open CompilationServer
+open CompilationCache
 open Type
 open Json
 
@@ -13,6 +13,7 @@ type server_message_options = {
 	mutable print_parsed : bool;
 	mutable print_removed_directory : bool;
 	mutable print_reusing : bool;
+	mutable print_retyping : bool;
 	mutable print_skipping_dep : bool;
 	mutable print_unchanged_content : bool;
 	mutable print_cached_modules : bool;
@@ -38,6 +39,7 @@ let config = {
 	print_parsed = false;
 	print_removed_directory = false;
 	print_reusing = false;
+	print_retyping = false;
 	print_skipping_dep = false;
 	print_unchanged_content = false;
 	print_cached_modules = false;
@@ -56,7 +58,7 @@ let config = {
 
 let sign_string com =
 	let sign = Define.get_signature com.defines in
-	let cs = CompilationServer.force () in
+	let cs = com.cs in
 	let	sign_id = (cs#get_context sign)#get_index in
 	Printf.sprintf "%2i,%3s: " sign_id (short_platform_name com.platform)
 
@@ -85,8 +87,17 @@ let removed_directory com tabs dir =
 let reusing com tabs m =
 	if config.print_reusing then print_endline (Printf.sprintf "%s%sreusing %s" (sign_string com) tabs (s_type_path m.m_path))
 
-let skipping_dep com tabs (m,path) =
-	if config.print_skipping_dep then print_endline (Printf.sprintf "%sskipping %s%s" (sign_string com) (s_type_path m.m_path) (if m.m_path = path then "" else Printf.sprintf "(%s)" (s_type_path path)))
+let retyper_ok com tabs m =
+	if config.print_retyping then print_endline (Printf.sprintf "%s%sretyped %s" (sign_string com) tabs (s_type_path m.m_path))
+
+let retyper_fail com tabs m reason =
+	if config.print_retyping then begin
+		print_endline (Printf.sprintf "%s%sfailed retyping %s" (sign_string com) tabs (s_type_path m.m_path));
+		print_endline (Printf.sprintf "%s%s%s" (sign_string com) (tabs ^ "  ") reason);
+	end
+
+let skipping_dep com tabs (m,reason) =
+	if config.print_skipping_dep then print_endline (Printf.sprintf "%sskipping %s (%s)" (sign_string com) (s_type_path m.m_path) reason)
 
 let unchanged_content com tabs file =
 	if config.print_unchanged_content then print_endline (Printf.sprintf "%s%s changed time not but content, reusing" (sign_string com) file)
@@ -105,8 +116,11 @@ let completion str =
 
 let defines com tabs =
 	if config.print_defines then begin
-		let defines = PMap.foldi (fun k v acc -> (k ^ "=" ^ v) :: acc) com.defines.Define.values [] in
-		print_endline ("Defines " ^ (String.concat "," (List.sort compare defines)))
+		let buffer = Buffer.create 64 in
+		Buffer.add_string buffer "Defines ";
+		PMap.iter (Printf.bprintf buffer "%s=%s,") com.defines.values;
+		Buffer.truncate buffer (Buffer.length buffer - 1);
+		print_endline (Buffer.contents buffer)
 	end
 
 let signature com tabs sign =
@@ -151,6 +165,7 @@ let enable_all () =
 	config.print_parsed <- true;
 	config.print_removed_directory <- true;
 	config.print_reusing <- true;
+	config.print_retyping <- true;
 	config.print_skipping_dep <- true;
 	config.print_unchanged_content <- true;
 	config.print_cached_modules <- true;
@@ -174,6 +189,7 @@ let set_by_name name value = match name with
 	| "parsed" -> config.print_parsed <- value;
 	| "removedDirectory" -> config.print_removed_directory <- value;
 	| "reusing" -> config.print_reusing <- value;
+	| "retyping" -> config.print_retyping <- value;
 	| "skippingDep" -> config.print_skipping_dep <- value;
 	| "unchangedContent" -> config.print_unchanged_content <- value;
 	| "cachedModules" -> config.print_cached_modules <- value;
@@ -187,4 +203,4 @@ let set_by_name name value = match name with
 	| "socketMessage" -> config.print_socket_message <- value;
 	| "uncaughtError" -> config.print_uncaught_error <- value;
 	| "newContext" -> config.print_new_context <- value;
-	| _ -> raise Not_found
\ No newline at end of file
+	| _ -> raise Not_found
