@@ -190,4 +190,24 @@ let report_times print =
 	in
 	List.iter (loop 0) root.children;
 	print sep;
-	print_time "total" root
\ No newline at end of file
+	print_time "total" root
+
+class timer (id : string list) = object(self)
+	method run_finally : 'a . (unit -> 'a) -> (unit -> unit) -> 'a = fun f finally ->
+		let timer = timer id in
+		try
+			let r = f() in
+			timer();
+			finally();
+			r
+		with exc ->
+			timer();
+			finally();
+			raise exc
+
+	method run : 'a . (unit -> 'a) -> 'a = fun f ->
+		self#run_finally f (fun () -> ())
+
+	method nest (name : string) =
+		new timer (id @ [name])
+end
\ No newline at end of file
