@@ -281,7 +281,7 @@ class typed_functions = object(self)
 end
 
 type typed_function_kind =
-	| FuncLocal
+	| FuncLocal of string option
 	| FuncMember of jpath * string
 	| FuncStatic of jpath * string
 
@@ -294,39 +294,17 @@ module JavaFunctionalInterfaces = struct
 		jparams : string list;
 	}
 
-	let java_functional_interfaces =
-		let juf = ["java";"util";"function"] in
-		let tp name = TTypeParameter name in
-		[
-			{
-				jargs = [];
-				jret = None;
-				jpath = ["java";"lang"],"Runnable";
-				jname = "run";
-				jparams = []
-			};
-			{
-				jargs = [tp "T"];
-				jret = None;
-				jpath = juf,"Consumer";
-				jname = "accept";
-				jparams = ["T"]
-			};
-			{
-				jargs = [tp "T";tp "U"];
-				jret = None;
-				jpath = juf,"BiConsumer";
-				jname = "accept";
-				jparams = ["T";"U"]
-			};
-			{
-				jargs = [tp "T"];
-				jret = Some (tp "R");
-				jpath = juf,"Function";
-				jname = "apply";
-				jparams = ["T";"R"]
-			};
-		]
+	let java_functional_interfaces = DynArray.create ()
+
+	let add args ret path name params =
+		let jfi = {
+			jargs = args;
+			jret = ret;
+			jpath = path;
+			jname = name;
+			jparams = params;
+		} in
+		DynArray.add java_functional_interfaces jfi
 
 	let unify jfi args ret =
 		let rec loop params want have = match want,have with
@@ -357,15 +335,22 @@ module JavaFunctionalInterfaces = struct
 			None
 
 
-	let find_compatible args ret =
-		ExtList.List.filter_map (fun jfi ->
-			if jfi.jparams = [] then begin
-				if jfi.jargs = args && jfi.jret = ret then
-					Some (jfi,[])
-				else None
+	let find_compatible args ret filter =
+		DynArray.fold_left (fun acc jfi ->
+			if filter = [] || List.mem jfi.jpath filter then begin
+				if jfi.jparams = [] then begin
+					if jfi.jargs = args && jfi.jret = ret then
+						(jfi,[]) :: acc
+					else
+						acc
+				end else match unify jfi args ret with
+					| Some x ->
+						x :: acc
+					| None ->
+						acc
 			end else
-				unify jfi args ret
-		) java_functional_interfaces
+				acc
+		) [] java_functional_interfaces
 end
 
 open JavaFunctionalInterfaces
@@ -382,8 +367,13 @@ class typed_function
 
 	val jc_closure =
 		let name = match kind with
-			| FuncLocal ->
-				Printf.sprintf "Closure_%s_%i" (patch_name host_method#get_name) host_method#get_next_closure_id
+			| FuncLocal s ->
+				let name = patch_name host_method#get_name in
+				let name = match s with
+					| None -> name
+					| Some s -> name ^ "_" ^ s
+				in
+				Printf.sprintf "Closure_%s_%i" name (host_class#get_next_closure_id name)
 			| FuncStatic(path,name) ->
 				Printf.sprintf "%s_%s" (snd path) (patch_name name)
 			| FuncMember(path,name) ->
@@ -406,7 +396,7 @@ class typed_function
 		jm_ctor#return;
 		jm_ctor
 
-	method generate_invoke (args : (string * jsignature) list) (ret : jsignature option)=
+	method generate_invoke (args : (string * jsignature) list) (ret : jsignature option) (functional_interface_filter : jpath list) =
 		let arg_sigs = List.map snd args in
 		let meth = functions#register_signature arg_sigs ret in
 		let jsig_invoke = method_sig arg_sigs ret in
@@ -419,14 +409,17 @@ class typed_function
 			end
 		in
 		let spawn_forward_function meth_from meth_to is_bridge =
-			let flags = [MPublic] in
-			let flags = if is_bridge then MBridge :: MSynthetic :: flags else flags in
-			let jm_invoke_next = jc_closure#spawn_method meth_from.name (method_sig meth_from.dargs meth_from.dret) flags in
-			functions#make_forward_method jc_closure jm_invoke_next meth_from meth_to;
+			let msig = method_sig meth_from.dargs meth_from.dret in
+			if not (jc_closure#has_method meth_from.name msig) then begin
+				let flags = [MPublic] in
+				let flags = if is_bridge then MBridge :: MSynthetic :: flags else flags in
+				let jm_invoke_next = jc_closure#spawn_method meth_from.name msig flags in
+				functions#make_forward_method jc_closure jm_invoke_next meth_from meth_to;
+			end
 		in
 		let check_functional_interfaces meth =
 			try
-				let l = JavaFunctionalInterfaces.find_compatible meth.dargs meth.dret in
+				let l = JavaFunctionalInterfaces.find_compatible meth.dargs meth.dret functional_interface_filter in
 				List.iter (fun (jfi,params) ->
 					add_interface jfi.jpath params;
 					spawn_forward_function {meth with name=jfi.jname} meth false;
