@@ -31,15 +31,28 @@ let rec pow a b = match b with
 	| _ -> Int32.mul a (pow a (b - 1))
 
 let java_hash s =
+	let high_surrogate c = (c lsr 10) + 0xD7C0 in
+	let low_surrogate c = (c land 0x3FF) lor 0xDC00 in
 	let h = ref Int32.zero in
-	let l = UTF8.length s in
-	let i31 = Int32.of_int 31 in
-	let i = ref 0 in
-	UTF8.iter (fun char ->
-		let char = Int32.of_int (UCharExt.uint_code char) in
-		h := Int32.add !h (Int32.mul char (pow i31 (l - (!i + 1))));
-		incr i;
-	) s;
+	let thirtyone = Int32.of_int 31 in
+	(try
+		UTF8.validate s;
+		UTF8.iter (fun c ->
+			let c = (UCharExt.code c) in
+			if c > 0xFFFF then
+				(h := Int32.add (Int32.mul thirtyone !h)
+					(Int32.of_int (high_surrogate c));
+				h := Int32.add (Int32.mul thirtyone !h)
+					(Int32.of_int (low_surrogate c)))
+			else
+				h := Int32.add (Int32.mul thirtyone !h)
+					(Int32.of_int c)
+			) s
+	with UTF8.Malformed_code ->
+		String.iter (fun c ->
+			h := Int32.add (Int32.mul thirtyone !h)
+				(Int32.of_int (Char.code c))) s
+	);
 	!h
 
 module HashtblList = struct
@@ -141,8 +154,8 @@ class builder jc name jsig = object(self)
 	val mutable stack_frames = []
 	val mutable exceptions = []
 	val mutable argument_locals = []
+	val mutable argument_annotations = Hashtbl.create 0
 	val mutable thrown_exceptions = Hashtbl.create 0
-	val mutable closure_count = 0
 	val mutable regex_count = 0
 
 	(* per-frame *)
@@ -191,11 +204,6 @@ class builder jc name jsig = object(self)
 			| _ -> JvmVerificationTypeInfo.of_signature jc#get_pool t
 		) locals
 
-	method get_next_closure_id =
-		let id = closure_count in
-		closure_count <- closure_count + 1;
-		id
-
 	method get_next_regex_id =
 		let id = regex_count in
 		regex_count <- regex_count + 1;
@@ -1002,6 +1010,10 @@ class builder jc name jsig = object(self)
 	method replace_top jsig =
 		code#get_stack#replace jsig
 
+	method add_argument_annotation (slot : int) (a : (path * annotation) list) =
+		let a = Array.of_list (List.map (fun (path,annot) -> TObject(path,[]),annot) a) in
+		Hashtbl.add argument_annotations slot a
+
 	(** This function has to be called once all arguments are declared. *)
 	method finalize_arguments =
 		argument_locals <- locals
@@ -1123,6 +1135,18 @@ class builder jc name jsig = object(self)
 		end;
 		if Hashtbl.length thrown_exceptions > 0 then
 			self#add_attribute (AttributeExceptions (Array.of_list (Hashtbl.fold (fun k _ c -> k :: c) thrown_exceptions [])));
+		if Hashtbl.length argument_annotations > 0 then begin
+			let l = List.length argument_locals in
+			let offset = if self#has_method_flag MStatic then 0 else 1 in
+			let a = Array.init (l - offset) (fun i ->
+				try
+					let annot = Hashtbl.find argument_annotations (i + offset) in
+					convert_annotations jc#get_pool annot
+				with Not_found ->
+					[||]
+			) in
+			DynArray.add attributes (AttributeRuntimeVisibleParameterAnnotations a)
+		end;
 		let attributes = self#export_attributes jc#get_pool in
 		let offset_name = jc#get_pool#add_string name in
 		let jsig = generate_method_signature false jsig in
@@ -1139,6 +1163,7 @@ class builder jc name jsig = object(self)
 		assert (code#get_fp = 0);
 		assert (not was_exported);
 		was_exported <- true;
+		self#commit_annotations jc#get_pool;
 		let attributes = self#export_attributes jc#get_pool in
 		let offset_name = jc#get_pool#add_string name in
 		let jsig = generate_signature false jsig in
