@@ -81,6 +81,7 @@ module NativeSignatures = struct
 	let java_class_sig = TObject(java_class_path,[TType(WNone,object_sig)])
 
 	let haxe_jvm_path = ["haxe";"jvm"],"Jvm"
+	let haxe_type_path = ["haxe";"root"],"Type"
 
 	let haxe_dynamic_object_path = ["haxe";"jvm"],"DynamicObject"
 	let haxe_dynamic_object_sig = TObject(haxe_dynamic_object_path,[])
@@ -118,6 +119,12 @@ module NativeSignatures = struct
 	let haxe_function_path = (["haxe";"jvm"],"Function")
 	let haxe_function_sig = TObject(haxe_function_path,[])
 
+	let haxe_compiled_pattern_path = (["haxe";"jvm"],"CompiledPattern")
+	let haxe_compiled_pattern_sig = TObject(haxe_compiled_pattern_path,[])
+
+	let haxe_ereg_path = (["haxe";"root"],"EReg")
+	let haxe_ereg_sig = TObject(haxe_ereg_path,[])
+
 	let void_path = ["java";"lang"],"Void"
 	let void_sig = TObject(void_path,[])
 
