@@ -43,6 +43,9 @@ module BasicBlock = struct
 		| BKUnreachable             (* The unique unreachable block *)
 		| BKCatch of tvar           (* A catch block *)
 
+	type flag_block =
+		| BlockDce
+
 	type cfg_edge_Flag =
 		| FlagExecutable      (* Used by constant propagation to handle live edges *)
 		| FlagDce             (* Used by DCE to keep track of handled edges *)
@@ -67,7 +70,7 @@ module BasicBlock = struct
 		| SEIfThenElse of t * t * t * Type.t * pos               (* `if` with "then", "else" and "next" *)
 		| SESwitch of (texpr list * t) list * t option * t * pos (* `switch` with cases, "default" and "next" *)
 		| SETry of t * t * (tvar * t) list * t *  pos            (* `try` with "exc", catches and "next" *)
-		| SEWhile of t * t * t * pos                             (* `while` with "head", "body" and "next" *)
+		| SEWhile of t * t * pos                                 (* `while` with "body" and "next" *)
 		| SESubBlock of t * t                                    (* "sub" with "next" *)
 		| SEMerge of t                                           (* Merge to same block *)
 		| SENone                                                 (* No syntax exit *)
@@ -93,6 +96,7 @@ module BasicBlock = struct
 		bb_pos : pos;                         (* The block position *)
 		bb_kind : block_kind;                 (* The block kind *)
 		mutable bb_closed : bool;             (* Whether or not the block has been closed *)
+		mutable bb_flags : int;
 		(* elements *)
 		bb_el : texpr DynArray.t;             (* The block expressions *)
 		bb_phi : texpr DynArray.t;            (* SSA-phi expressions *)
@@ -115,6 +119,18 @@ module BasicBlock = struct
 		| LUEl of int
 		| LUTerm
 
+	let int_of_block_flag (flag : flag_block) =
+		Obj.magic flag
+
+	let add_block_flag bb (flag : flag_block) =
+		bb.bb_flags <- set_flag bb.bb_flags (int_of_block_flag flag)
+
+	let remove_block_flag bb (flag : flag_block) =
+		bb.bb_flags <- unset_flag bb.bb_flags (int_of_block_flag flag)
+
+	let has_block_flag bb (flag : flag_block) =
+		has_flag bb.bb_flags (int_of_block_flag flag)
+
 	let s_block_kind = function
 		| BKRoot -> "BKRoot"
 		| BKNormal -> "BKNormal"
@@ -177,6 +193,7 @@ module BasicBlock = struct
 			bb_type = t;
 			bb_pos = p;
 			bb_closed = false;
+			bb_flags = 0;
 			bb_el = DynArray.create();
 			bb_phi = DynArray.create();
 			bb_terminator = TermNone;
@@ -556,9 +573,8 @@ module Graph = struct
 					loop scopes' bb_exc;
 					List.iter (fun (_,bb_catch) -> loop (next_scope scopes) bb_catch) catches;
 					loop scopes bb_next
-				| SEWhile(bb_head,bb_body,bb_next,_) ->
+				| SEWhile(bb_body,bb_next,_) ->
 					let scopes' = next_scope scopes in
-					loop scopes' bb_head;
 					loop scopes' bb_body;
 					loop scopes bb_next;
 				| SESubBlock(bb_sub,bb_next) ->
@@ -578,10 +594,13 @@ type analyzer_context = {
 	config : AnalyzerConfig.t;
 	graph : Graph.t;
 	temp_var_name : string;
+	with_timer : 'a . string list -> (unit -> 'a) -> 'a;
+	identifier : string;
 	mutable entry : BasicBlock.t;
 	mutable has_unbound : bool;
 	mutable loop_counter : int;
 	mutable loop_stack : int list;
 	mutable debug_exprs : (string * texpr) list;
 	mutable name_stack : string list;
+	mutable did_optimize : bool;
 }
